@@ -100,6 +100,10 @@ MainView *interface_main_view_new(AppData * data)
         result->can_show_node_view = FALSE;
         result->node_list_longpress_path = NULL;
         result->checklist_longpress_source_id = 0;
+        result->node_view_prev_keyval = GDK_VoidSymbol;
+        result->node_view_prev_keyval_reset = FALSE;
+        result->main_view_prev_keyval = GDK_VoidSymbol;
+        result->main_view_prev_keyval_reset = FALSE;
 
 	GtkWidget *main_vbox = gtk_vbox_new(FALSE, 0);
 
@@ -168,8 +172,13 @@ MainView *interface_main_view_new(AppData * data)
 
         /* Signals on the main view */
         g_signal_connect(G_OBJECT(result->data->main_view), "key-press-event", G_CALLBACK(on_main_view_key_press), result);
+        g_signal_connect(G_OBJECT(result->data->main_view), "key-release-event", G_CALLBACK(on_main_view_key_release), result);
 	g_signal_connect(G_OBJECT(result->data->main_view), "delete_event", G_CALLBACK(close_cb), result);
 
+        /* Signals on the node view */
+        g_signal_connect(G_OBJECT(result->data->node_view), "key-press-event", G_CALLBACK(on_node_view_key_press), result);
+        g_signal_connect(G_OBJECT(result->data->node_view), "key-release-event", G_CALLBACK(on_node_view_key_release), result);
+
 	sk_set_brushsize(result, 2);
 	sk_set_brushsize(result, 4);
 
@@ -308,17 +317,28 @@ GtkWidget *create_menu(MainView * main)
     main->menu_button_delete = hildon_button_new_with_text(0, 0, _("Delete memo"), NULL);
     main->menu_button_export = hildon_button_new_with_text(0, 0, _("Export to file"), NULL);
     main->menu_button_clear = hildon_button_new_with_text(0, 0, _("Clear contents"), NULL);
+
+    /* Sketch-specific */
     main->menu_button_square = hildon_check_button_new(0);
     gtk_button_set_label(GTK_BUTTON(main->menu_button_square), _("Square shapes"));
     main->menu_button_filled = hildon_check_button_new(0);
     gtk_button_set_label(GTK_BUTTON(main->menu_button_filled), _("Fill shapes"));
 
+    /* Rich text-specific */
+    main->menu_button_wordwrap = hildon_check_button_new(0);
+    gtk_button_set_label(GTK_BUTTON(main->menu_button_wordwrap), _("Word wrap"));
+
+    /* Checklist-specific */
+    main->menu_button_remove_checked = hildon_button_new_with_text(0, 0, _("Remove checked items"), NULL);
+
     g_signal_connect(G_OBJECT(main->menu_button_rename), "clicked", G_CALLBACK(callback_file_rename_node), main);
     g_signal_connect(G_OBJECT(main->menu_button_delete), "clicked", G_CALLBACK(callback_file_delete_node), main);
     g_signal_connect(G_OBJECT(main->menu_button_export), "clicked", G_CALLBACK(callback_file_export_node), main);
     g_signal_connect(G_OBJECT(main->menu_button_clear), "clicked", G_CALLBACK(callback_edit_clear), main);
     g_signal_connect(G_OBJECT(main->menu_button_square), "toggled", G_CALLBACK(callback_sketch_button_toggled), main);
     g_signal_connect(G_OBJECT(main->menu_button_filled), "toggled", G_CALLBACK(callback_sketch_button_toggled), main);
+    g_signal_connect(G_OBJECT(main->menu_button_wordwrap), "toggled", G_CALLBACK(callback_wordwrap), main);
+    g_signal_connect(G_OBJECT(main->menu_button_remove_checked), "clicked", G_CALLBACK(callback_remove_checked), main);
 
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_rename));
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_delete));
@@ -326,6 +346,8 @@ GtkWidget *create_menu(MainView * main)
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_clear));
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_square));
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_filled));
+    hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_wordwrap));
+    hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_remove_checked));
 
     hildon_window_set_app_menu(HILDON_WINDOW(main->data->node_view),
             HILDON_APP_MENU(main->node_view_app_menu));
@@ -365,7 +387,7 @@ GtkWidget *create_menu(MainView * main)
 	main->rename_node_item = gtk_menu_item_new_with_label(_("Rename memo"));
 	main->export_node_item = gtk_menu_item_new_with_label(_("Export memo"));
 
-	main->move_node_item = gtk_menu_item_new_with_label(_("Move node"));
+	main->move_node_item = gtk_menu_item_new_with_label(_("Move memo"));
 	main->move_up_node_item = gtk_menu_item_new_with_label(_("Move up"));
 	main->move_down_node_item = gtk_menu_item_new_with_label(_("Move down"));
 	main->move_to_bottom_node_item = gtk_menu_item_new_with_label(_("Move to bottom"));
@@ -409,7 +431,6 @@ GtkWidget *create_menu(MainView * main)
 	main->tools_shape = gtk_menu_item_new_with_label(_("Shape"));
 	main->tools_color = gtk_menu_item_new_with_label(_("Color..."));
 	main->tools_font = gtk_menu_item_new_with_label(_("Font..."));
-	main->tools_wordwrap = gtk_check_menu_item_new_with_label(_("Word wrap"));
 	main->tools_pressure = gtk_check_menu_item_new_with_label(_("Pressure sensitive brush color"));
 
 	gtk_menu_append(tools_menu, main->tools_color);
@@ -417,7 +438,6 @@ GtkWidget *create_menu(MainView * main)
 	gtk_menu_append(tools_menu, main->tools_shape);
 	gtk_menu_append(tools_menu, main->tools_pagestyle);
 	gtk_menu_append(tools_menu, main->tools_font);
-	gtk_menu_append(tools_menu, main->tools_wordwrap);
 	gtk_menu_append(tools_menu, main->tools_pressure);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main->tools_brushsize), main->brushsizemenu);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main->tools_pagestyle), main->sketchlinesmenu);
@@ -436,7 +456,6 @@ GtkWidget *create_menu(MainView * main)
 	g_signal_connect(G_OBJECT(main->move_to_bottom_node_item), "activate", G_CALLBACK(callback_move_to_bottom_level_node), main);
 	
 	g_signal_connect(G_OBJECT(main->tools_font), "activate", G_CALLBACK(callback_font), main);
-	g_signal_connect(G_OBJECT(main->tools_wordwrap), "toggled", G_CALLBACK(callback_wordwrap), main);
 	g_signal_connect(G_OBJECT(main->tools_pressure), "toggled", G_CALLBACK(callback_pressure), main);
 	g_signal_connect(G_OBJECT(main->tools_color), "activate", G_CALLBACK(callback_color_invoke), main);
 
@@ -653,6 +672,7 @@ static void create_toolbar(MainView * main)
 	g_signal_connect(G_OBJECT(main->colorbutton_tb), "clicked", G_CALLBACK(callback_color), main);
 
 	main->brushsize_tb = gtk_tool_button_new(NULL, NULL);
+	_tool_button_set_icon_file(GTK_TOOL_BUTTON(main->brushsize_tb), PIXMAPDIR "/pencil1.png");
 	g_signal_connect(G_OBJECT(main->brushsize_tb), "clicked", G_CALLBACK(callback_brushsizetb), main);
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->brushsize_tb, -1);
 
@@ -662,16 +682,23 @@ static void create_toolbar(MainView * main)
 	g_signal_connect(G_OBJECT(main->eraser_tb), "toggled", G_CALLBACK(callback_eraser), main);
 
 	main->shape_tb = gtk_tool_button_new(NULL, NULL);
+	_tool_button_set_icon_file(GTK_TOOL_BUTTON(main->shape_tb), PIXMAPDIR "/shape0.png");
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->shape_tb, -1);
 	g_signal_connect(G_OBJECT(main->shape_tb), "clicked", G_CALLBACK(callback_menu), main->shapemenu);
 
 	main->sketchlines_tb = gtk_tool_button_new(NULL, NULL);
+	_tool_button_set_icon_file(GTK_TOOL_BUTTON(main->sketchlines_tb), PIXMAPDIR "/sketchbg0.png");
 	g_signal_connect(G_OBJECT(main->sketchlines_tb), "clicked", G_CALLBACK(callback_menu), main->sketchlinesmenu);
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->sketchlines_tb, -1);
 
 	
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), gtk_separator_tool_item_new(), -1);
 
+	main->sharing_tb = gtk_tool_button_new(NULL, NULL);
+	_tool_button_set_icon_name(GTK_TOOL_BUTTON(main->sharing_tb), "control_sharing");
+	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->sharing_tb, -1);
+	g_signal_connect(G_OBJECT(main->sharing_tb), "clicked", G_CALLBACK(callback_sharing), main);
+
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->checkadd_tb, -1);
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->checkedit_tb, -1);
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->checkdel_tb, -1);
@@ -821,7 +848,7 @@ void create_treeview(MainView * main)
 	gtk_tree_view_column_pack_start(column, renderer2, FALSE);
 	gtk_tree_view_column_pack_start(column, renderer1, TRUE);
 
-	gtk_tree_view_column_add_attribute(column, renderer1, "text", NODE_NAME);
+	gtk_tree_view_column_add_attribute(column, renderer1, "markup", NODE_NAME);
 	gtk_tree_view_column_add_attribute(column, renderer2, "pixbuf", NODE_PIXBUF);
 
         /* Ellipsize to avoid horizontal scroll bars */
@@ -844,11 +871,7 @@ void create_treeview(MainView * main)
 	gtk_tree_selection_set_mode(select, GTK_SELECTION_BROWSE);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(main->treeview), FALSE);
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(main->treeview), FALSE);
-#ifdef EXPANDING_ROWS
 	gtk_tree_view_set_expander_column(GTK_TREE_VIEW(main->treeview), column);
-#else
-	gtk_tree_view_set_expander_column(GTK_TREE_VIEW(main->treeview), NULL);
-#endif
 
 	gtk_tree_view_expand_all(GTK_TREE_VIEW(main->treeview));
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(main->treeview), TRUE);
@@ -857,9 +880,6 @@ void create_treeview(MainView * main)
 	g_signal_connect(G_OBJECT(main->treeview), "button-press-event", G_CALLBACK(callback_treeview_button_press), main);
 
 	g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(callback_treeview_change), main);
-#ifndef EXPANDING_ROWS
-	g_signal_connect(G_OBJECT(main->treeview), "test-collapse-row", G_CALLBACK(callback_treeview_testcollapse), main);
-#endif
 	gtk_tree_selection_set_select_function(select, treeview_canselect, main, NULL);
 	gtk_object_set_user_data(GTK_OBJECT(main->treeview), (gpointer) main);
 
